import { useAsync } from '@react-devui/hooks';
import { isPrintableCharacter } from './utils';
export function DComboboxKeyboardSupport(props) {
    const { children, dVisible, dEditable = false, dNested = false, onVisibleChange, onFocusChange } = props;
    const asyncCapture = useAsync();
    return children({
        ksOnKeyDown: (e) => {
            let code = e.code;
            if (code === 'Escape') {
                e.preventDefault();
                onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(false);
            }
            else {
                if (dVisible) {
                    if (!dNested && ['ArrowLeft', 'ArrowRight'].includes(code)) {
                        code = null;
                    }
                    switch (code) {
                        case 'ArrowUp':
                            e.preventDefault();
                            onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('prev');
                            break;
                        case 'ArrowDown':
                            e.preventDefault();
                            onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('next');
                            break;
                        case 'Home':
                            e.preventDefault();
                            onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('first');
                            break;
                        case 'End':
                            e.preventDefault();
                            onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('last');
                            break;
                        case 'ArrowLeft':
                            e.preventDefault();
                            onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('prev-level');
                            break;
                        case 'ArrowRight':
                            e.preventDefault();
                            onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('next-level');
                            break;
                        default:
                            break;
                    }
                }
                else {
                    if (dEditable && ['Home', 'End', 'Enter', 'Space'].includes(code)) {
                        code = null;
                    }
                    switch (code) {
                        case 'ArrowUp':
                            e.preventDefault();
                            onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(true);
                            asyncCapture.requestAnimationFrame(() => onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('last'));
                            break;
                        case 'ArrowDown':
                            e.preventDefault();
                            onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(true);
                            asyncCapture.requestAnimationFrame(() => onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('first'));
                            break;
                        case 'Home':
                            e.preventDefault();
                            onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(true);
                            asyncCapture.requestAnimationFrame(() => onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('first'));
                            break;
                        case 'End':
                            e.preventDefault();
                            onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(true);
                            asyncCapture.requestAnimationFrame(() => onFocusChange === null || onFocusChange === void 0 ? void 0 : onFocusChange('last'));
                            break;
                        case 'Enter':
                        case 'Space':
                            e.preventDefault();
                            onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(true);
                            break;
                        default:
                            if (isPrintableCharacter(e.key)) {
                                onVisibleChange === null || onVisibleChange === void 0 ? void 0 : onVisibleChange(true);
                            }
                            break;
                    }
                }
            }
        },
    });
}
//# sourceMappingURL=ComboboxKeyboardSupport.js.map