import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useElement, useImmer } from '@react-devui/hooks';
import { usePrefixConfig } from '../../hooks';
import { DToast, ToastService, toastSubject } from '../toast';
export function Toast() {
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    const [toasts, setToasts] = useImmer(new Map());
    const getRoot = (id) => {
        let root = document.getElementById(`${dPrefix}toast-root`);
        if (!root) {
            root = document.createElement('div');
            root.id = `${dPrefix}toast-root`;
            document.body.appendChild(root);
        }
        let el = document.getElementById(id);
        if (!el) {
            el = document.createElement('div');
            el.id = id;
            root.appendChild(el);
        }
        return el;
    };
    const toastTRoot = useElement(() => getRoot(`${dPrefix}toast-t-root`));
    const toastBRoot = useElement(() => getRoot(`${dPrefix}toast-b-root`));
    useEffect(() => {
        const obs = [];
        const mergeProps = (uniqueId, props) => {
            return {
                onClose: () => {
                    var _a;
                    (_a = props.onClose) === null || _a === void 0 ? void 0 : _a.call(props);
                    ToastService.close(uniqueId);
                },
                afterVisibleChange: (visible) => {
                    var _a;
                    (_a = props.afterVisibleChange) === null || _a === void 0 ? void 0 : _a.call(props, visible);
                    if (!visible) {
                        setToasts((draft) => {
                            draft.delete(uniqueId);
                        });
                    }
                },
            };
        };
        obs.push(toastSubject.open$.subscribe({
            next: ({ uniqueId, props }) => {
                setToasts((draft) => {
                    draft.set(uniqueId, Object.assign(Object.assign(Object.assign({}, props), { dVisible: true }), mergeProps(uniqueId, props)));
                });
            },
        }), toastSubject.close$.subscribe({
            next: (uniqueId) => {
                setToasts((draft) => {
                    const props = draft.get(uniqueId);
                    if (props) {
                        draft.set(uniqueId, Object.assign(Object.assign({}, props), { dVisible: false }));
                    }
                });
            },
        }), toastSubject.rerender$.subscribe({
            next: ({ uniqueId, props: newProps }) => {
                setToasts((draft) => {
                    const props = draft.get(uniqueId);
                    if (props) {
                        draft.set(uniqueId, Object.assign(Object.assign(Object.assign({}, newProps), { dVisible: props.dVisible }), mergeProps(uniqueId, newProps)));
                    }
                });
            },
        }), toastSubject.closeAll$.subscribe({
            next: (animation) => {
                setToasts((draft) => {
                    if (animation) {
                        for (const props of draft.values()) {
                            props.dVisible = false;
                        }
                    }
                    else {
                        draft.clear();
                    }
                });
            },
        }));
    }, [setToasts]);
    return (_jsxs(_Fragment, { children: [toastTRoot &&
                ReactDOM.createPortal(Array.from(toasts.entries())
                    .filter(([, toastProps]) => { var _a; return ((_a = toastProps.dPlacement) !== null && _a !== void 0 ? _a : 'top') === 'top'; })
                    .map(([uniqueId, toastProps]) => _jsx(DToast, Object.assign({}, toastProps), uniqueId)), toastTRoot), toastBRoot &&
                ReactDOM.createPortal(Array.from(toasts.entries())
                    .filter(([, toastProps]) => toastProps.dPlacement === 'bottom')
                    .map(([uniqueId, toastProps]) => _jsx(DToast, Object.assign({}, toastProps), uniqueId)), toastBRoot)] }));
}
//# sourceMappingURL=Toast.js.map