import type { DId } from '../../utils';
export declare type TreeNodeStatus = 'INDETERMINATE' | 'CHECKED' | 'UNCHECKED';
export declare abstract class AbstractTreeNode<ID extends DId, T> {
    origin: T;
    private _parent;
    constructor(origin: T);
    get parent(): AbstractTreeNode<ID, T> | null;
    get root(): AbstractTreeNode<ID, T>;
    get isLeaf(): boolean;
    get id(): ID;
    get status(): TreeNodeStatus;
    get checked(): boolean;
    get unchecked(): boolean;
    get indeterminate(): boolean;
    get disabled(): boolean;
    get enabled(): boolean;
    updateStatus(checked: ID | null | Set<ID>, onlySelf?: boolean): void;
    protected _setParent(parent: AbstractTreeNode<ID, T>): void;
    protected _updateAncestors(checked: ID | null | Set<ID>, onlySelf: boolean): void;
    abstract children?: AbstractTreeNode<ID, T>[];
    protected abstract _id: ID;
    protected abstract _status: TreeNodeStatus;
    protected abstract _disabled: boolean;
    protected abstract _calculateStatus(checked: ID | null | Set<ID>): void;
}
export declare class SingleTreeNode<ID extends DId, T extends {
    disabled?: boolean;
    children?: T[];
}> extends AbstractTreeNode<ID, T> {
    private getId;
    private opts;
    children?: SingleTreeNode<ID, T>[];
    protected _id: ID;
    protected _status: TreeNodeStatus;
    protected _disabled: boolean;
    constructor(origin: T, getId: (origin: T) => ID, opts: {
        disabled?: boolean;
    });
    private _setUpChildren;
    protected _calculateStatus(checked: ID | null): void;
}
export declare class MultipleTreeNode<ID extends DId, T extends {
    disabled?: boolean;
    children?: T[];
}> extends AbstractTreeNode<ID, T> {
    private getId;
    private opts;
    children?: MultipleTreeNode<ID, T>[];
    protected _id: ID;
    protected _status: TreeNodeStatus;
    protected _disabled: boolean;
    constructor(origin: T, getId: (origin: T) => ID, opts: {
        disabled?: boolean;
    });
    changeStatus(status: TreeNodeStatus, checkeds: Set<ID>): Set<ID>;
    protected _calculateStatus(checkeds: Set<ID>): void;
    protected _changeStatus(status: TreeNodeStatus, initValue: Set<ID>): Set<ID>;
    private _setUpChildren;
}
